<?php

namespace Drupal\webform_donate\Element;

use Drupal\webform\Element\WebformCompositeBase;
use Drupal\webform_donate\Plugin\WebformElement\WebformDonate as WebformDonateElement;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\webform\Entity\Webform;
use Drupal\media_entity\Entity\Media;
use Drupal\currency\Entity\Currency;

/**
 * Provides a webform element for an donations amount element.
 *
 * @FormElement("webform_donate")
 */
class WebformDonate extends WebformCompositeBase {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    return parent::getInfo();
  }

  /**
   * {@inheritdoc}
   */
  public static function getCompositeElements(array $element) {
    $elements = [];
    $elements['frequency'] = [
      '#type' => 'radios',
      '#title' => t('Frequency'),
      '#options' => 'frequency',
      '#weight' => 0,
    ];

    $elements['amount'] = [
      '#type' => 'textfield',
      '#title' => t('Amount'),
      '#weight' => 1,
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public static function processWebformComposite(&$element, FormStateInterface $form_state, &$complete_form) {
    parent::processWebformComposite($element, $form_state, $complete_form);
    $currency = Currency::load('GBP');

    if (!empty($element['#webform']) && $webform = Webform::load($element['#webform'])) {

      $frequencies = WebformDonateElement::getFrequencies(FALSE);
      $amounts = WebformDonateElement::getAmounts($frequencies, $element);
      $amount_defaults = WebformDonateElement::getAmountDefaults($frequencies, $element);

      // Set frequency options.
      $element['frequency'] = array_merge($element['frequency'], WebformDonate::getElementProperties($amounts));
      $element['frequency']['#default_value'] = $amount_defaults['frequency_default'];

      // Hide frequency selection if one or no options.
      if (count($element['frequency']['#options']) <= 1) {
        $element['frequency']['#access'] = FALSE;
      }

      $element['amount']['#type'] = 'container';
      $element['#element_validate'] = [
        [
          get_called_class(),
          'validateWebformDonate',
        ],
      ];

      foreach ($amounts as $frequency_id => $frequency_details) {
        $element['amount'][$frequency_id] = [
          '#type' => 'container',
          '#attributes' => [
            'class' => [
              'hide-element',
            ],
          ],
          '#states' => array(
            'visible' => array(
              ":input[name='" . $element['#name'] . "[frequency]']" => array('value' => $frequency_id),
            ),
          ),
        ];

        $benefits = WebformDonate::getElementProperties($frequency_details['amounts'], 'benefit');
        $options = $benefits['#options'];
        if (!empty($frequency_details['amounts'])) {
          $element['amount'][$frequency_id]['amounts'] = [
            '#type' => 'webform_buttons',
            '#title' => $element['amount']['#title'],
            '#default_value' => key($options),
            '#options' => $options,
          ];
        }

        if (!empty($frequency_details['allow_other_amount'])) {
          $element['amount'][$frequency_id]['amounts']['#type'] = 'webform_buttons_other';
          $element['amount'][$frequency_id]['amounts']['#other__type'] = 'number';
          $element['amount'][$frequency_id]['amounts']['#other__option_label'] = 'Other';
          $element['amount'][$frequency_id]['amounts']['#other__field_prefix'] = $currency->getSign();
          if (!empty($frequency_details['minimum_amount'])) {
            $element['amount'][$frequency_id]['amounts']['#other__min'] = $frequency_details['minimum_amount'];
          }
        }

        if (!empty($frequency_details['allow_hero_image']) && !empty($frequency_details['hero_image_reference'])) {
          $media_entity = Media::load($frequency_details['hero_image_reference']);
          if (!empty($media_entity)) {
            $element['amount'][$frequency_id]['hero_image'] = [
              '#theme' => 'image_style',
              '#style_name' => $frequency_details['hero_image_style'],
              '#uri' => $media_entity->field_image->entity->getFileUri(),
              '#theme_wrappers' => [
                'container' => [
                  '#attributes' => ['class' => []],
                ],
              ],
              '#weight' => 10,
            ];
          }
        }

        // Add a benefit message element.
        foreach ($benefits['#button_descriptions'] as $id => $button_description) {
          $element['amount'][$frequency_id][$id] = [
            '#type' => 'container',
            '#attributes' => [
              'class' => [
                'hide-element',
              ],
            ],
            '#states' => array(
              'visible' => array(
                ":input[name='" . $element['#name'] . "[amount][" . $frequency_id . "][amounts][buttons]']" => array('value' => $id),
              ),
            ),
          ];

          $element['amount'][$frequency_id][$id]['message'] = [
            '#type' => 'webform_markup',
            '#markup' => $button_description,
          ];
        }
      }
    }

    return $element;
  }

  /**
   * Get element properties from a settings array.
   *
   * @param array $settings
   *   Settings array.
   * @param string $description_key
   *   Key.
   *
   * @return array
   *   Return element properties array.
   */
  private static function getElementProperties(array $settings, $description_key = '') {
    $options = [];
    $descriptions = [];

    if (!empty($settings)) {
      $default_key = array_keys($settings)[0];
      foreach ($settings as $key => $details) {
        $options[$key] = $details['label'];

        if ($description_key) {
          $descriptions[$key] = !empty($details[$description_key]) ? $details[$description_key] : '';
        }
      }
    }

    return [
      '#options' => $options,
      '#button_descriptions' => $descriptions,
    ];
  }

  /**
   * Validates a webform_donate_amount element.
   */
  public static function validateWebformDonate(&$element, FormStateInterface $form_state, &$complete_form) {
    // Donation webform amount field must be converted into a single value.
    $frequency = $element['frequency']['#value'] ?: $element['frequency']['#default_value'];
    $frequency = trim($frequency);
    $form_state->set('frequency', $frequency);

    $amount = NestedArray::getValue($form_state->getValues(), [
      $element['#name'],
      'amount',
      $frequency,
      'amounts',
    ]);
    $form_state->setValueForElement($element['amount'], $amount);
    $form_state->set('amount', $amount);

    return $element;
  }

}
